/********************
 * 2004 Jalview Reengineered
 * Barton Group
 * Dundee University
 *
 * AM Waterhouse
 *******************/


package jalview.gui;

import jalview.gui.*;
import jalview.io.*;
import jalview.datamodel.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import javax.swing.*;
import java.awt.datatransfer.*;
import jalview.bin.Cache;


public class Desktop extends jalview.jbgui.GDesktop implements DropTargetListener
{
  public static JDesktopPane desktop;
  static int openFrameCount = 0;
  static final int xOffset = 30, yOffset = 30;

  public Desktop()
  {
    Image image =null;
    try{
      java.net.URL url = getClass().getResource("/images/logo.gif");
      if(url!=null)
      {
        image = java.awt.Toolkit.getDefaultToolkit().createImage(url);
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 0);
        mt.waitForID(0);
        setIconImage(image);
      }

    }catch(Exception ex){}

    setTitle("Jalview 2005");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    desktop = new JDesktopPane();
    desktop.setBackground(Color.white);
    setContentPane(desktop);
    desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);


    // This line prevents Windows Look&Feel resizing all new windows to maximum
    // if previous window was maximised
    desktop.setDesktopManager( new DefaultDesktopManager() );


    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    String x = jalview.bin.Cache.getProperty("SCREEN_X");
    String y = jalview.bin.Cache.getProperty("SCREEN_Y");
    String width = jalview.bin.Cache.getProperty("SCREEN_WIDTH");
    String height = jalview.bin.Cache.getProperty("SCREEN_HEIGHT");

    if(x!=null && y!=null && width!=null && height!=null)
    {
      setBounds( Integer.parseInt(x), Integer.parseInt(y),
                 Integer.parseInt(width), Integer.parseInt(height));
    }
    else
      setBounds( (int)(screenSize.width-900)/2,
           (int)(screenSize.height-650)/2,
          900 ,
          650);

this.addWindowListener(new WindowAdapter()
{
public void windowClosing(WindowEvent evt)
{
 jalview.bin.Cache.setProperty("SCREEN_X",
                               getBounds().x + "");
 jalview.bin.Cache.setProperty("SCREEN_Y",
                               getBounds().y + "");
 jalview.bin.Cache.setProperty("SCREEN_WIDTH", getWidth() + "");
 jalview.bin.Cache.setProperty("SCREEN_HEIGHT", getHeight() + "");
}
});
setVisible(true);

this.setDropTarget(new java.awt.dnd.DropTarget(desktop, this));

/////////Add a splashscreen on startup
   /////////Add a splashscreen on startup
    JInternalFrame frame = new JInternalFrame();

    SplashScreen splash = new SplashScreen(frame, image);
    frame.setContentPane(splash);
    frame.setLayer(JLayeredPane.PALETTE_LAYER);
    addInternalFrame(frame, "", 750,160, false);
    frame.setLocation( (int)((getWidth()-750) /2), (int)((getHeight()-160) /2));
  }
  public static void addInternalFrame(final JInternalFrame frame, String title, int w, int h)
  {
    addInternalFrame(frame, title, w, h, true);
  }

  public static void addInternalFrame(final JInternalFrame frame, String title,
                                      int w,
                                      int h,
                                      boolean resizable )
  {
   desktop.add(frame);
   openFrameCount++;
   try {
       frame.setSelected(true);
   } catch (java.beans.PropertyVetoException e) {}
   frame.setTitle(title);
   frame.setSize(w,h);
   frame.setClosable(true);
   frame.setResizable(resizable);
   frame.setMaximizable(resizable);
   frame.setIconifiable(resizable);
   frame.setFrameIcon(null);
   frame.setLocation(xOffset*openFrameCount, yOffset*openFrameCount);
   frame.toFront();
   final JMenuItem menuItem = new JMenuItem(title);
   frame.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter()
   {
     public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
     {
       openFrameCount--;
       windowMenu.remove(menuItem);
     };
   });

   menuItem.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try{  frame.setSelected(true);   frame.setIcon(false); }
        catch(java.beans.PropertyVetoException ex){};
      }
    });

   frame.setVisible(true);
   windowMenu.add(menuItem);
 }

 public void dragEnter(DropTargetDragEvent evt){}
 public void dragExit(DropTargetEvent evt){}
 public void dragOver(DropTargetDragEvent evt){}
 public void dropActionChanged(DropTargetDragEvent evt){}
 public void drop(DropTargetDropEvent evt)
{
   Transferable t = evt.getTransferable();
   if(!t.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
     return;

   evt.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
   try{
     java.util.List files = (java.util.List) t.getTransferData(DataFlavor.javaFileListFlavor);

     for (int i = 0; i < files.size(); i++)
     {
       String file = files.get(i).toString();
       String protocol = "File";
       if(file.endsWith(".jar"))
                Jalview2XML.LoadJalviewAlign(file);
      else
      {
        String format = jalview.io.IdentifyFile.Identify(file, protocol);
        LoadFile(file, protocol, format);
      }
     }

   }catch(Exception ex){ex.printStackTrace();}
  }


 public void inputLocalFileMenuItem_actionPerformed(ActionEvent e)
 {
   JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"),
       new String[]{"fa, fasta, fastq", "aln",  "pfam", "msf", "pir","blc","jar"},
       new String[]{"Fasta", "Clustal", "PFAM", "MSF", "PIR", "BLC", "Jalview Archive"}
       ,null);

   chooser.setFileView(new JalviewFileView());
   chooser.setDialogTitle("Open local file");
   chooser.setToolTipText("Open");
   int value = chooser.showOpenDialog(this);
   if(value == JalviewFileChooser.APPROVE_OPTION)
   {
     String choice =  chooser.getSelectedFile().getPath();
     jalview.bin.Cache.setProperty("LAST_DIRECTORY", chooser.getSelectedFile().getParent());
     if(chooser.getSelectedFormat().equals("Jalview"))
     {
       Jalview2XML.LoadJalviewAlign(choice);
     }
     else
     {
       String format = IdentifyFile.Identify(choice, "File");
       LoadFile(choice, "File", format);
     }
   }
 }

 public void LoadFile(String file, String protocol,  String format)
 {
   LoadingThread loader = new LoadingThread(file, protocol, format);
   loader.start();
 }

 class LoadingThread extends Thread
 {
   String file, protocol, format;

   public LoadingThread(String file, String protocol,  String format)
   {
     this.file = file;
     this.protocol = protocol;
     this.format = format;
   }
   public void run()
   {
     SequenceI [] sequences = null;

     if (FormatAdapter.formats.contains(format))
         sequences = FormatAdapter.readFile(file, protocol, format);

     if (sequences != null && sequences.length>0)
     {
       AlignFrame af = new AlignFrame(new Alignment(sequences));
       addInternalFrame(af, file, AlignFrame.NEW_WINDOW_WIDTH, AlignFrame.NEW_WINDOW_HEIGHT);
       af.currentFileFormat = format;
       af.statusBar.setText("Successfully loaded file " + file);
       try{
           af.setMaximum(Preferences.showFullscreen);
       }catch(Exception ex){}

     }
     else
       JOptionPane.showInternalMessageDialog(Desktop.desktop,  "Couldn't open file.\n"
                            + "Formats currently supported are\n"
                            + "Fasta, MSF, Clustal, BLC, PIR, MSP, and PFAM" // JBPNote - message should be generated through FormatAdapter!
                            ,"Error loading file",
                                     JOptionPane.WARNING_MESSAGE);

   }
 }


 public void inputURLMenuItem_actionPerformed(ActionEvent e)
 {
   String url = JOptionPane.showInternalInputDialog(Desktop.desktop,"Enter url of input file",
                                            "Input alignment from URL",
                                            JOptionPane.QUESTION_MESSAGE,
                                            null, null,
                                            "http://www.").toString();
   if (url == null)
     return;

   String format = IdentifyFile.Identify(url, "URL");

   if (format.equals("URL NOT FOUND"))
   {
     JOptionPane.showInternalMessageDialog(Desktop.desktop,"Couldn't locate " + url,
                                   "URL not found",
                                   JOptionPane.WARNING_MESSAGE);
     return;
   }

    LoadFile(url, "URL", format);
 }

 public void inputTextboxMenuItem_actionPerformed(ActionEvent e)
 {
   CutAndPasteTransfer cap = new CutAndPasteTransfer(true);
   int accept =  JOptionPane.showInternalOptionDialog(Desktop.desktop, cap, "Cut & paste Alignment File",
                                       JOptionPane.YES_NO_CANCEL_OPTION,
                                       JOptionPane.PLAIN_MESSAGE,
                                       null,
                                       new Object[]{"Accept", "Cancel", }, null);


  if(accept == JOptionPane.YES_OPTION)
  {
    String format = IdentifyFile.Identify(cap.getText(), "Paste");
    SequenceI [] sequences = null;

    if (FormatAdapter.formats.contains( format ))
      sequences = FormatAdapter.readFile(cap.getText(), "Paste", format);


      if(sequences!=null)
      {
        AlignFrame af = new AlignFrame(new Alignment(sequences));
        af.currentFileFormat = format;
        addInternalFrame(af, "Cut & Paste input - "+format,
                         AlignFrame.NEW_WINDOW_WIDTH,
                         AlignFrame.NEW_WINDOW_HEIGHT);
        af.statusBar.setText("Successfully pasted alignment file");
      }
      else
        JOptionPane.showInternalMessageDialog(Desktop.desktop, "Couldn't read the pasted text.\n"
                                      +"Formats currently supported are\n"
                                      +"Fasta, MSF, Clustal, BLC, PIR, MSP, and PFAM",
                                      "Error parsing text", JOptionPane.WARNING_MESSAGE);

   }
 }

/*
* Exit the program
*/
 public void quit_actionPerformed(ActionEvent e)
 {
   if(jalview.bin.Jalview.applet!=null)
     jalview.bin.Jalview.applet.destroy();
    else
      System.exit(0);
 }



 public void aboutMenuItem_actionPerformed(ActionEvent e)
 {
   JOptionPane.showInternalMessageDialog(Desktop.desktop,
                                         "JalView 2005 version " + jalview.bin.Cache.VERSION+"; last updated: "+jalview.bin.Cache.BUILD_DATE
                                       +"\nAuthors:  Michele Clamp, James Cuff, Steve Searle, Andrew Waterhouse, Jim Procter & Geoff Barton."
                                       +"\nCurrent development managed by Andrew Waterhouse; Barton Group, University of Dundee."
                                       +"\nIf  you use JalView, please cite:"
                                       +"\n\"Clamp, M., Cuff, J., Searle, S. M. and Barton, G. J. (2004), The Jalview Java Alignment Editor\""
                                       +"\nBioinformatics,  2004 12;426-7.",
                                         "About Jalview",
                                         JOptionPane.INFORMATION_MESSAGE);
 }

 public void documentationMenuItem_actionPerformed(ActionEvent e)
 {
   try
   {
      ClassLoader cl = jalview.gui.Desktop.class.getClassLoader();
      java.net.URL url = javax.help.HelpSet.findHelpSet(cl,"help/help");
      javax.help.HelpSet hs = new javax.help.HelpSet(cl,url);

      javax.help.HelpBroker hb = hs.createHelpBroker();
      hb.setLocation(new Point(200, 50));
      hb.setSize(new Dimension(800,700));
      hb.setCurrentID("home");
      hb.setDisplayed(true);
   }
   catch (Exception ex)
   {
     ex.printStackTrace();
   }
 }

 protected void preferences_actionPerformed(ActionEvent e)
 {
   Preferences pref = new Preferences();
 }

 public void saveState_actionPerformed(ActionEvent e)
 {
   JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.
       getProperty("LAST_DIRECTORY"),
       new String[]
       {"xml"}, new String[]{"Jalview Project"}, "Jalview Project");

   chooser.setFileView(new JalviewFileView());
   chooser.setDialogTitle("Save State");
   int value = chooser.showSaveDialog(this);
   if (value == JalviewFileChooser.APPROVE_OPTION)
   {
     java.io.File choice = chooser.getSelectedFile();
     jalview.bin.Cache.setProperty("LAST_DIRECTORY", choice.getParent());
     Jalview2XML.SaveState(choice);
   }

 }


 public void loadState_actionPerformed(ActionEvent e)
 {
   JalviewFileChooser chooser = new JalviewFileChooser(jalview.bin.Cache.getProperty("LAST_DIRECTORY"),
       new String[]{"xml"}, new String[]{"Jalview Project"}, "Jalview Project");
   chooser.setFileView(new JalviewFileView());
   chooser.setDialogTitle("Restore state");
   int value = chooser.showOpenDialog(this);
   if(value == JalviewFileChooser.APPROVE_OPTION)
   {
     String choice =  chooser.getSelectedFile().getAbsolutePath();
     jalview.bin.Cache.setProperty("LAST_DIRECTORY", chooser.getSelectedFile().getParent());
     Jalview2XML.LoadState(choice);
   }
 }
}


